﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Xml;
using System.Xml.Linq;
using System.IO;

namespace Client
{
    public static class XLinqExtensions
    {
        internal static bool HasClass(this XElement element, string className)
        {
            var attr = element.Attribute("class");
            return (attr != null &&
                    !string.IsNullOrEmpty(attr.Value) &&
                    attr.Value.Equals(className, StringComparison.InvariantCultureIgnoreCase));
        }

        public static IEnumerable<XElement> Divs(this XElement element, string className)
        {
            return element.Elements(XHtml.Div).Where(div => div.HasClass(className));
        }

        public static XElement Div(this XElement element, string className)
        {
            return Divs(element, className).First();
        }

        public static IEnumerable<XElement> Spans(this XElement element, string className)
        {
            return element.Elements(XHtml.Span).Where(span => span.HasClass(className));
        }

        public static XElement Span(this XElement element, string className)
        {
            return Spans(element, className).First();
        }

        public static IEnumerable<XElement> Ols(this XElement element, string className)
        {
            return element.Elements(XHtml.Ol).Where(ol => ol.HasClass(className));
        }

        public static XElement Ol(this XElement element, string className)
        {
            return Ols(element, className).First();
        }

        public static IEnumerable<XElement> Uls(this XElement element, string className)
        {
            return element.Elements(XHtml.Ul).Where(ul => ul.HasClass(className));
        }

        public static XElement Ul(this XElement element, string className)
        {
            return Uls(element, className).First();
        }

        public static IEnumerable<XElement> Lis(this XElement element)
        {
            return element.Elements(XHtml.Li);
        }

        public static XHtmlForm Form(this XElement element, string className)
        {
            return new XHtmlForm(element.Elements(XHtml.Form).Where(form => form.HasClass(className)).First());
        }

        public static XHtmlAnchor Anchor(this XElement element)
        {
            return new XHtmlAnchor(element.Elements(XHtml.A).First());
        }

        public static XHtmlAnchor Anchor(this XElement element, string className)
        {
            return new XHtmlAnchor(element.Elements(XHtml.A).Where(a => a.HasClass(className)).First());
        }

        public static bool HasAnchor(this XElement element, string className)
        {
            return element.Elements(XHtml.A).First().HasClass(className);
        }

        public static XHtmlList List(this XElement element, string className)
        {
            return new XHtmlList(element.Elements(XHtml.Ol).Where(list => list.HasClass(className)).First());
        }

        public static XHtmlStruct Struct(this XElement element, string className)
        {
            return new XHtmlStruct(element.Elements(XHtml.Div).Where(_struct => _struct.HasClass(className)).First());
        }

        public static XElement Html(this XDocument doc)
        {
            return doc.Elements(XHtml.Html).First();
        }

        public static XElement Html(this XElement element)
        {
            return element.Document.Html();
        }
        
        public static XElement Body(this XDocument doc)
        {
            return doc.Html().Elements(XHtml.Body).First();
        }

        public static XElement Body(this XElement element)
        {
            return element.Document.Body();
        }
    }

    public class XHtmlStruct : IEnumerable<XElement>
    {
        XElement _struct;

        internal XHtmlStruct(XElement _struct)
        {
            this._struct = _struct;
        }

        public XElement this[string className]
        {
            get
            {
                return _struct.Descendants().First(element => element.HasClass(className));
            }
        }

        public XHtmlAnchor Anchor(string className)
        {
            return _struct.Anchor(className);
        }


        #region IEnumerable<XElement> Members

        public IEnumerator<XElement> GetEnumerator()
        {
            foreach (var element in _struct.Elements())
            {
                yield return element;
            }
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            foreach (var element in _struct.Elements())
            {
                yield return element;
            }
        }

        #endregion
    }

    public class XHtmlList : IEnumerable<XElement>
    {
        XElement list;

        internal XHtmlList(XElement list)
        {
            this.list = list;
        }

        public XElement this[int index]
        {
            get
            {
                return list.Elements().ElementAt(index);
            }
        }

        public int Count
        {
            get
            {
                return list.Elements().Count();
            }
        }

        #region IEnumerable<XElement> Members

        public IEnumerator<XElement> GetEnumerator()
        {
            foreach (var element in list.Elements())
            {
                yield return element;
            }
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            foreach (var element in list.Elements())
            {
                yield return element;
            }
        }

        #endregion
    }

    public class XHtmlForm
    {
        XElement _form;
        Dictionary<string, string> _inputs = new Dictionary<string, string>();
        Dictionary<string, string> _hiddenInputs = new Dictionary<string, string>();

        internal XHtmlForm(XElement form)
        {
            _form = form;

            _form.Descendants(XHtml.Input).Where(input =>
            {
                var type = input.Attribute("type");
                return type != null
                       && !string.IsNullOrEmpty(type.Value)
                       && (type.Value.Equals("text", StringComparison.InvariantCultureIgnoreCase)
                           || type.Value.Equals("submit", StringComparison.InvariantCultureIgnoreCase)
                           || type.Value.Equals("hidden", StringComparison.InvariantCultureIgnoreCase)
                           || type.Value.Equals("password", StringComparison.InvariantCultureIgnoreCase));
            }).Select(input =>
                new { Type = input.Attribute("type").Value,
                      Name = input.Attribute("name").Value,
                      Value = input.Attribute("value") == null ? "" : input.Attribute("value").Value }).Each(input =>
                {
                    if (input.Type != "hidden")
                    {
                        _inputs.Add(input.Name, input.Value ?? "");
                    }
                    else
                    {
                        _hiddenInputs.Add(input.Name, input.Value ?? "");
                    }
                });
        }

        public string this[string name]
        {
            get
            {
                return _inputs[name];
            }
            set
            {
                if (_inputs.Keys.Contains(name))
                {
                    _inputs[name] = value ?? "";
                }
                else
                {
                    throw new ArgumentException(string.Format("{0} not present in form input", name));
                }
            }
        }

        internal string WwwEncodedFormUrlEncoded()
        {
            var encodedInput = _inputs.Keys.Select(key => string.Format("{0}={1}", key, HttpUtility.UrlEncode(_inputs[key])));
            var encodedHiddenInput = _hiddenInputs.Keys.Select(key => string.Format("{0}={1}", key, HttpUtility.UrlEncode(_hiddenInputs[key])));
            return string.Join("&", encodedInput.Union(encodedHiddenInput).ToArray());
        }

        public XDocument Submit()
        {
            XDocument result = null;

            //Console.WriteLine("Submit with...");
            //_inputs.Keys.OfType<string>().Each(key => Console.WriteLine("{0} => {1}", key, _inputs[key]));
            
            var formParams = WwwEncodedFormUrlEncoded();
            //Console.WriteLine(formParams);

            var action = _form.Attribute("action").Value;
            var method = _form.Attribute("method").Value;

            // need to remove _method parameter if present
            // and convert method to that
            Regex regex = new Regex(@"(\?|\&)_method=(put|post|delete)$");
            Match match = regex.Match(action.ToLower());
            if (match.Success)
            {
                // method is in group 2
                method = match.Groups[2].Value;

                // remove entire _method token from URI
                action = action.Substring(0, match.Index);
            }

            if (method.Equals("get", StringComparison.InvariantCultureIgnoreCase))
            {
                Uri baseUri = _form.Document.Annotation<Uri>();
                Uri actionUri = new Uri(baseUri, string.Format("{0}{1}{2}",
                    action,
                    action.Contains('?') ? '&' : '?',
                    formParams));
                //Console.WriteLine(actionUri);

                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(actionUri.ToString());
                if (_form.Document.Annotation<CookieContainer>() != null)
                    req.CookieContainer = _form.Document.Annotation<CookieContainer>();
                req.Method = method;
                HttpWebResponse resp = (HttpWebResponse)req.GetResponse();
                result = XDocument.Load(XmlReader.Create(resp.GetResponseStream(), new XmlReaderSettings { CloseInput = true, ProhibitDtd = false }));
                result.AddAnnotation(actionUri);
            }
            else // put | post | delete
            {
                Uri baseUri = _form.Document.Annotation<Uri>();
                Uri actionUri = new Uri(baseUri, action);
                //Console.WriteLine(actionUri);
                
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(actionUri.ToString());
                if (_form.Document.Annotation<CookieContainer>() != null)
                    req.CookieContainer = _form.Document.Annotation<CookieContainer>();
                req.Method = method;
                req.ContentType = @"application/x-www-form-urlencoded";
                using (var tw = new StreamWriter(req.GetRequestStream()))
                {
                    tw.WriteLine(formParams);
                }
                HttpWebResponse resp = (HttpWebResponse)req.GetResponse();
                result = XDocument.Load(XmlReader.Create(resp.GetResponseStream(), new XmlReaderSettings { CloseInput = true, ProhibitDtd = false }));
                result.AddAnnotation(actionUri);
            }

            return result;
        }
    }

    public class XHtmlAnchor
    {
        XElement _anchor;

        internal XHtmlAnchor(XElement anchor)
        {
            _anchor = anchor;
        }

        public XDocument Navigate()
        {
            XDocument result = null;

            var href = _anchor.Attribute("href").Value;
  
            Uri baseUri = _anchor.Document.Annotation<Uri>();
            Uri hrefUri = new Uri(baseUri, href);
            //Console.WriteLine(hrefUri);

            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(hrefUri.ToString());
            if (_anchor.Document.Annotation<CookieContainer>() != null)
                req.CookieContainer = _anchor.Document.Annotation<CookieContainer>();
            HttpWebResponse resp = (HttpWebResponse)req.GetResponse();
            result = XDocument.Load(XmlReader.Create(resp.GetResponseStream(), new XmlReaderSettings { CloseInput = true, ProhibitDtd = false }));
            result.AddAnnotation(hrefUri);

            return result;
        }

        public string AnchorText
        {
            get { return _anchor.Value; }
        }

        public string AnchorLink
        {
            get { return _anchor.Attribute("href").Value; }
        }

        public string Id
        {
            get { return _anchor.Attribute("id") == null ? "" : _anchor.Attribute("id").Value; }
        }
    }
}
