using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using MvcBookmarkService.Models;

namespace MvcBookmarkService.Controllers
{
    [HandleError]
    public class BookmarkController : Controller
    {
        BookmarksRepository bmRepository = new BookmarksRepository();

        //
        // GET: /bookmarks/

        public ActionResult BookmarkIndex()
        {
            var bms = bmRepository.FindAllPublicBookmarks().ToList();
            return View("BookmarkIndex", bms);
        }

        //
        // GET: /users/{username}

        public ActionResult BookmarksByUserIndex(string username)
        {
            List<Bookmark> bms = null;
            if (username.Equals(HttpContext.User.Identity.Name))
                bms = bmRepository.FindBookmarksByUser(username).ToList();
            else
                bms = bmRepository.FindPublicBookmarksByUser(username).ToList();
            ViewData["username"] = username;
            return View("BookmarksByUserIndex", bms);
        }

        //
        // GET: /tags/{tag}

        public ActionResult BookmarksByTagIndex(string tag)
        {
            List<Bookmark> bms = bmRepository.FindBookmarksByTag(tag).ToList();
            ViewData["tag"] = tag;
            return View("BookmarksByTagIndex", bms);
        }

        //
        // GET: /bookmarks/details/{id}

        public ActionResult Details(int id)
        {
            var bm = bmRepository.FindBookmarkById(id);
            if (bm == null)
                throw new HttpException(404, "Not Found");

            if (!bm.Shared)
            {
                if (!bm.Username.Equals(HttpContext.User.Identity.Name))
                    return new HttpUnauthorizedResult();
            }
            return View("Details", bm);
        }

        //
        // GET: /bookmarks/delete/{id}

        [Authorize]
        public ActionResult Delete(int id)
        {
            var bm = bmRepository.FindBookmarkById(id);
            if (bm == null)
                throw new HttpException(404, "Not Found");

            if (!bm.Username.Equals(HttpContext.User.Identity.Name))
                return new HttpUnauthorizedResult();
            return View("Delete", bm);
        }

        //
        // DELETE/POST: /bookmarks/delete/{id}

        [Authorize]
        [AcceptVerbs(HttpVerbs.Delete | HttpVerbs.Post)]
        public ActionResult Delete(int id, FormCollection collection)
        {
            try
            {
                var bm = bmRepository.FindBookmarkById(id);
                if (bm == null)
                    throw new HttpException(404, "Not Found");

                if (!bm.Username.Equals(HttpContext.User.Identity.Name))
                    return new HttpUnauthorizedResult();

                bmRepository.DeleteBookmark(bm);
                bmRepository.Save();
                return RedirectToAction("BookmarksByUserIndex", new { username = HttpContext.User.Identity.Name });
            }
            catch
            {
                return View();
            }
        }


        //
        // GET: /bookmarks/create

        [Authorize]
        public ActionResult Create()
        {
            return View("Create");
        } 

        //
        // POST: /bookmarks/create

        [Authorize]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(FormCollection collection)
        {
            try
            {
                Bookmark bm = new Bookmark();
                bm.Title = collection["bookmark-title"];
                bm.Url = collection["bookmark-url"];
                bm.Shared = collection["bookmark-shared"].Contains("true");
                bm.LastModified = DateTime.Now;
                bm.Username = HttpContext.User.Identity.Name;
                bm.Tags = collection["bookmark-tags"];

                bmRepository.AddBookmark(bm);
                bmRepository.Save();
                
                string tags = collection["bookmark-tags"];
                string[] tagsList = tags.Split(',');
                foreach (string t in tagsList)
                    bmRepository.AddTagForBookmark(t, bm);
                
                return RedirectToAction("BookmarksByUserIndex", new { username = HttpContext.User.Identity.Name });
            }
            catch
            {
                return View();
            }
        }

        //
        // GET: /bookmarks/edit/{id}

        [Authorize]
        public ActionResult Edit(int id)
        {
            var bm = bmRepository.FindBookmarkById(id);
            if (bm == null)
                throw new HttpException(404, "Not Found");

            if (!bm.Username.Equals(HttpContext.User.Identity.Name))
                return new HttpUnauthorizedResult();
            return View("Edit", bm);
        }

        //
        // PUT/POST: /bookmarks/edit/{id}

        [Authorize]
        [AcceptVerbs(HttpVerbs.Put | HttpVerbs.Post)]
        public ActionResult Edit(int id, FormCollection collection)
        {
            try
            {
                var bm = bmRepository.FindBookmarkById(id);
                if (bm == null)
                    throw new HttpException(404, "Not Found");

                if (!bm.Username.Equals(HttpContext.User.Identity.Name))
                    return new HttpUnauthorizedResult();

                bm.Title = collection["bookmark-title"];
                bm.Url = collection["bookmark-url"];
                bm.Tags = collection["bookmark-tags"];
                bm.Shared = collection["bookmark-shared"].Contains("true");
                bm.LastModified = DateTime.Now;
                bmRepository.Save();

                string tags = collection["bookmark-tags"];
                string[] tagsList = tags.Split(',');
                foreach (string t in tagsList)
                    bmRepository.AddTagForBookmark(t, bm);
                bmRepository.Save();

                return RedirectToAction("Details", new { id = bm.BookmarkID });
            }
            catch
            {
                return View();
            }
        }

        // GET: /users

        public ActionResult UserIndex()
        {
            var users = bmRepository.FindUsers().ToList();
            return View("UserIndex", users);
        }

        // GET: /tags

        public ActionResult TagIndex()
        {
            var tags = bmRepository.FindTags().ToList();
            return View("TagIndex", tags);
        }

     }
}
